/* all comment are related to the stylable js runtime mode no need to do anything here yet! */

import { wrapSBRender } from './wrap-render';
import { Stylesheet } from "stylable";
import { StylableComponent, SBStatelessComponent, StateLess, SBComponentProps } from "./types";

function SBComponent<T extends React.ComponentClass<any>>(Base: T, stylesheet: StylesheetInput): StylableComponent<T>;
function SBComponent<T>(stylesheet: StylesheetInput): Function;
function SBComponent<T extends React.ComponentClass<any>>(Base: T | StylesheetInput, stylesheet?: StylesheetInput) {
    if (!stylesheet) {
        return function (Component: T): StylableComponent<T> {
            return SBComponent(Component, Base as Stylesheet);
        }
    }
    const sheet =  (<{$stylesheet: Stylesheet}>stylesheet).$stylesheet || stylesheet;
    const Class = Base as any;
    Class.prototype.render = wrapSBRender(Class.prototype.render, sheet);
    return Class;
}

function SBStateless<T, C = object>(renderFunction: StateLess<T, C>, stylesheet: StylesheetInput): SBStatelessComponent<T> {
    const sheet =  (<{$stylesheet: Stylesheet}>stylesheet).$stylesheet || stylesheet;
    const wrapped = wrapSBRender(renderFunction as StateLess<T, C>, sheet);
    return wrapped;
}

function defineMixin<T>(name: string, mixinFunction: (options: T) => object) {
    const mixinId = "@Mixins/" + name;
    (mixinFunction as any).toString = function () { return mixinId; }
    return mixinFunction;
}

export type StylesheetInput = Stylesheet | {$stylesheet: Stylesheet};
export { SBComponent, SBStateless, defineMixin };

declare module 'react' {
    interface HTMLAttributes<T> extends SBComponentProps { }
}

